"use strict";
/// <reference types="greensock" />
/// <reference types="createjs" />
/// <reference types="preloadjs" />
/// <reference types="soundjs" />
Object.defineProperty(exports, "__esModule", { value: true });
var ElearningPlayerBridge = /** @class */ (function () {
    function ElearningPlayerBridge(assetsManifest) {
        var _this = this;
        /**
         * Start assets loading (TODO: add more initializations)
         */
        this.start = function () {
            //console.log("[ElearningPlayerBridge] start");
            _this.load();
        };
        this.init = function () {
            _this.onReady();
            _this.updateTimelineInstance();
            //ao iniciar a página, seta o status para "playing";
            _this.playing = true;
        };
        this.updateTimelineInstance = function () {
            //holding all page animations on a timeline
            _this.timeline = window.TimelineLite ? window.TimelineLite.exportRoot() : null;
        };
        /**
         * Play sounds based on id
         */
        this.playSound = function (id) {
            _this.sound = createjs.Sound.play(id);
            _this.sound.volume = _this.volume;
            _this.dispatchEventWith(ElearningPlayerBridge.ON_SOUND_STATE_CHANGE);
        };
        /**
         * Play sounds based on id
         */
        this.setSubtitle = function (value) {
            //console.log("[ElearningPlayerBridge] setSubtitle");
            _this.subtitle = value;
            _this.dispatchEventWith(ElearningPlayerBridge.ON_SUBTITLE, value);
        };
        /**
         * Must be used to define the page custom code;
         * Users must override this method or listen to this event;
         */
        this.onReady = function () {
            //console.log("[ElearningPlayerBridge] onReady");
            // dispatch a on_ready event.
            _this.dispatchEventWith(ElearningPlayerBridge.ON_READY);
        };
        /**
         * Page status toggle
         */
        this.togglePlayPause = function () {
            //console.log("[ElearningPlayerBridge] togglePlayPause");
            //TODO: pausar o som;
            if (_this.playing) {
                _this.pause();
            }
            else {
                _this.resume();
            }
        };
        /**
         * Toggle volume
         */
        this.toggleVolume = function () {
            //console.log("[ElearningPlayerBridge] toggleVolume");
            if (_this.sound) {
                _this.sound.volume = _this.sound.volume == 1 ? 0 : 1;
                _this.volume = _this.sound.volume;
                _this.dispatchEventWith(ElearningPlayerBridge.ON_SOUND_VOLUME_CHANGE);
            }
        };
        /**
         * Execute resume on animation instances (TimelineLite), sound instances (SoundJS), change page status state;
         */
        this.resume = function () {
            //console.log("[ElearningPlayerBridge] resume");
            if (_this.timeline)
                _this.timeline.resume();
            if (_this.sound) {
                _this.sound.paused = false;
                _this.dispatchEventWith(ElearningPlayerBridge.ON_SOUND_STATE_CHANGE);
            }
            _this.playing = true;
            _this.dispatchEventWith(ElearningPlayerBridge.ON_RESUME);
        };
        /**
         * Execute pause on animation instances (TimelineLite), sound instances (SoundJS), change page status state;
         */
        this.pause = function () {
            //console.log("[ElearningPlayerBridge] pause");
            _this.updateTimelineInstance();
            if (_this.timeline)
                _this.timeline.pause();
            if (_this.sound) {
                _this.sound.paused = true;
                _this.dispatchEventWith(ElearningPlayerBridge.ON_SOUND_STATE_CHANGE);
            }
            _this.playing = false;
            _this.dispatchEventWith(ElearningPlayerBridge.ON_PAUSE);
        };
        /**
         * Page assets loading;
         */
        this.load = function () {
            if (_this.manifest && _this.manifest.length > 0) {
                _this.preload = new createjs.LoadQueue(true);
                _this.preload.installPlugin(createjs.Sound);
                _this.preload.on("fileload", _this.handleFileLoad);
                _this.preload.on("progress", _this.handleOverallProgress);
                _this.preload.on("error", _this.handleFileError);
                while (_this.manifest.length > 0) {
                    _this.loadAnother();
                }
            }
            else {
                //Informa sobre o carregamento;
                _this.dispatchEventWith(ElearningPlayerBridge.ON_LOAD_PROGRESS, 1);
                //this.init();
            }
        };
        this.loadAnother = function () {
            // Get the next manifest item, and load it
            var item = _this.manifest.shift();
            _this.preload.loadFile(item);
            if (_this.manifest.length == 0) {
                //console.log("all assets included on stack");
            }
        };
        // File complete handler
        this.handleFileLoad = function (event) {
            //console.log("file loaded: ", event.item.id);
            var item = event.item;
            switch (item.type) {
                case "sound":
                    break;
                case "image":
                    break;
            }
            //console.log("file loaded result: ", event.result);
            if (_this.preload.progress >= 1) {
                _this.dispatchEventWith(ElearningPlayerBridge.ON_LOAD_PROGRESS, 1);
                _this.preload.off("fileload", _this.handleFileLoad);
                _this.preload.off("progress", _this.handleOverallProgress);
                _this.preload.off("error", _this.handleFileError);
            }
        };
        /**
         * Inform player about overall progress;
         */
        this.handleOverallProgress = function (event) {
            if (_this.preload.progress < 1) {
                _this.dispatchEventWith(ElearningPlayerBridge.ON_LOAD_PROGRESS, _this.preload.progress);
            }
        };
        //EVENT DISPATCHER
        this.addEventListener = function (event, callback) {
            // Check if the callback is not a function
            if (typeof callback !== 'function') {
                console.log("The listener callback must be a function, the given type is " + typeof callback);
                return;
            }
            // Check if the event is not a string
            if (typeof event !== 'string') {
                console.log("The event name must be a string, the given type is " + typeof event);
                return;
            }
            // Create the event if not exists
            if (_this.events[event] === undefined) {
                _this.events[event] = { listeners: [] };
            }
            _this.events[event].listeners.push(callback);
        };
        this.removeEventListener = function (event, callback) {
            // Check if this event not exists
            if (_this.events[event] === undefined) {
                console.log("This event: " + event + " does not exist");
                return;
            }
            _this.events[event].listeners = _this.events[event].listeners.filter(function (listener) {
                return listener.toString() !== callback.toString();
            });
        };
        this.dispatchEventWith = function (event, details) {
            //console.log("[ElearningPlayerBridge] dispatchEventWith: ", event);
            // Check if this event not exists
            if (_this.events[event] === undefined) {
                console.log("This event: " + event + " does not exist");
                return;
            }
            _this.events[event].listeners.forEach(function (listener) {
                listener(details);
            });
        };
        //console.log("[ElearningPlayerBridge] constructor");
        //assets list manifest;
        this.manifest = assetsManifest;
        //events list
        this.events = {};
        this.volume = 1;
    }
    Object.defineProperty(ElearningPlayerBridge.prototype, "sound", {
        get: function () {
            return this._sound;
        },
        set: function (value) {
            this._sound = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ElearningPlayerBridge.prototype, "subtitle", {
        get: function () {
            return this._subtitle;
        },
        set: function (value) {
            this._subtitle = value;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Clean up;
     */
    ElearningPlayerBridge.prototype.kill = function () {
        this.subtitle = "";
        this.playing = false;
        if (this.sound) {
            this.sound.destroy();
            this.dispatchEventWith(ElearningPlayerBridge.ON_SOUND_DESTROY);
        }
        if (this.timeline)
            this.timeline.clear();
        if (this.preload)
            this.preload.destroy();
    };
    // An error happened on a file
    ElearningPlayerBridge.prototype.handleFileError = function (event) {
        //console.log("error loading: ", event.item.id);
    };
    Object.defineProperty(ElearningPlayerBridge.prototype, "manifest", {
        get: function () {
            return this._manifest;
        },
        set: function (value) {
            this._manifest = value;
        },
        enumerable: true,
        configurable: true
    });
    ElearningPlayerBridge.ON_READY = "on_ready";
    ElearningPlayerBridge.ON_LOAD_PROGRESS = "on_load_progress";
    ElearningPlayerBridge.ON_SUBTITLE = "on_subtitle";
    ElearningPlayerBridge.ON_RESUME = "on_resume";
    ElearningPlayerBridge.ON_PAUSE = "on_pause";
    ElearningPlayerBridge.ON_SOUND_VOLUME_CHANGE = "on_sound_volume_change";
    ElearningPlayerBridge.ON_SOUND_STATE_CHANGE = "on_sound_state_change";
    ElearningPlayerBridge.ON_SOUND_DESTROY = "on_sound_destroy";
    return ElearningPlayerBridge;
}());
exports.ElearningPlayerBridge = ElearningPlayerBridge;
